@@ -32,10 +32,6 @@
 
 class KAction;
 
-#ifdef QAPT_ENABLED
-class QAptActions;
-#endif
-
 /**
  * This class serves as a shared Main Window implementation that connects
  * all the various backend bits so that they don't have to be reimplemented
@@ -52,11 +48,11 @@ public:
     MuonMainWindow();
 
     QSize sizeHint() const;
-    bool isConnected();
-    void setActionsEnabled(bool enabled = true);
+    bool isConnected() const;
 
 Q_SIGNALS:
     void shouldConnect(bool isConnected);
+    void actionsEnabledChanged(bool enabled);
 
 protected slots:
     bool queryExit();
@@ -65,14 +61,10 @@ protected slots:
 public slots:
     void easterEggTriggered();
     void setCanExit(bool canExit);
+    virtual void setActionsEnabled(bool enabled=true);
 
 private:
     bool m_canExit;
-
-protected:
-#ifdef QAPT_ENABLED
-    QAptActions * const  m_actions;
-#endif
 };
 
 #endif
