@@ -27,5 +27,8 @@ AbstractResourcesBackend::AbstractResourcesBackend(QObject* parent)
 
 void AbstractResourcesBackend::installApplication(AbstractResource* app)
 {
-    installApplication(app, QHash<QString, bool>());
+    installApplication(app, AddonList());
 }
+
+void AbstractResourcesBackend::integrateMainWindow(MuonMainWindow*)
+{}
